"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Endpoint = void 0;
const options_1 = require("./options");
const qs_1 = __importDefault(require("qs"));
const url_join_1 = __importDefault(require("url-join"));
class Endpoint {
    setLocalOptions(options) {
        this.localOptions = options;
        return this;
    }
    // Allowing global options to configure all Endpoint classes.
    // Optionally override options with this.localOptions via this.setLocalOptions
    get options() {
        var _a;
        return (_a = this.localOptions) !== null && _a !== void 0 ? _a : options_1.globalOptions;
    }
    // Used in merging together AxiosRequestConfig values.
    // These are considered fallback values that
    // can be overridden by calling config.
    get config() {
        return {};
    }
    // This contains the protocol and domain, aka location.origin.
    // e.g. https://example.com, http://localhost:3000, etc.
    get origin() {
        return this.options.origin;
    }
    // This is the middle of the url path, usually a group prefix.
    // e.g. api/v1, user/settings, etc.
    get path() {
        return this.options.path;
    }
    // This is the last part of the url path, usually the resource name.
    // e.g. "user" or "profile"
    get resource() {
        return '';
    }
    // Used to set baseURL property in AxiosRequestConfig.
    get baseURL() {
        return (0, url_join_1.default)(this.origin, this.path, this.resource);
    }
    get stringifyOptions() {
        return { arrayFormat: 'brackets' };
    }
    get paramsSerializer() {
        return (params) => qs_1.default.stringify(params, this.stringifyOptions);
    }
    // Returns AxiosRequestConfig passed to axios.request.
    requestConfig(requestConfig) {
        var _a, _b;
        const config = Object.assign(Object.assign({}, this.config), requestConfig);
        config.baseURL = (_a = config.baseURL) !== null && _a !== void 0 ? _a : this.baseURL;
        config.headers = Object.assign(Object.assign({}, this.config.headers), config.headers);
        config.paramsSerializer = (_b = config.paramsSerializer) !== null && _b !== void 0 ? _b : this.paramsSerializer;
        return config;
    }
    // General request method that is used by all HTTP calls.
    request(requestConfig) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.options.axios.request(this.requestConfig(requestConfig));
            }
            catch (error) {
                return this.handleError(error);
            }
        });
    }
    handleError(error) {
        return __awaiter(this, void 0, void 0, function* () {
            throw error;
        });
    }
}
exports.Endpoint = Endpoint;
//# sourceMappingURL=endpoint.js.map