"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SessionCsrfCookieMixin = void 0;
const axios_1 = __importDefault(require("axios"));
const helpers_1 = require("../helpers");
const url_join_1 = __importDefault(require("url-join"));
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
function SessionCsrfCookieMixin(superClass) {
    return class extends superClass {
        constructor() {
            super(...arguments);
            /**
             * Used to tell if a csrf cookie fix request is happening.
             * Otherwise we'd have an infinite loop. This also ensures
             * we only retry once.
             *
             * @type {boolean}
             * @protected
             */
            this.csrfCookieFix = false;
        }
        // Decides if current response is a CSRF token mismatch error.
        // Override method as needed.
        isCsrfTokenMismatch(response) {
            const { message } = (0, helpers_1.safeResponseData)(response);
            return response.status === 419 && message === 'CSRF token mismatch.';
        }
        get csrfPath() {
            return (0, url_join_1.default)(this.path, 'sanctum/csrf-cookie');
        }
        // Requests new CSRF Cookie from common Laravel Sanctum endpoint.
        // Override method as needed.
        requestCsrfCookie() {
            return __awaiter(this, void 0, void 0, function* () {
                return axios_1.default.get(this.csrfPath, {
                    baseURL: this.origin,
                });
            });
        }
        // General request method that is used by all HTTP calls.
        request(requestConfig) {
            const _super = Object.create(null, {
                request: { get: () => super.request }
            });
            return __awaiter(this, void 0, void 0, function* () {
                const response = yield _super.request.call(this, requestConfig);
                if (!this.csrfCookieFix && this.isCsrfTokenMismatch(response)) {
                    this.csrfCookieFix = true;
                    try {
                        yield this.requestCsrfCookie();
                    }
                    catch (error) {
                        return response;
                    }
                    return _super.request.call(this, requestConfig);
                }
                return response;
            });
        }
    };
}
exports.SessionCsrfCookieMixin = SessionCsrfCookieMixin;
//# sourceMappingURL=sessionCsrfCookie.js.map